Class: Vmpooler::PoolManager::Provider::Base

Inherits:
Object
  • Object
show all
Defined in:
lib/vmpooler/providers/base.rb

Direct Known Subclasses

Dummy, VSphere

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(config, logger, metrics, name, options) ⇒ Base

Returns a new instance of Base.



17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
# File 'lib/vmpooler/providers/base.rb', line 17

def initialize(config, logger, metrics, name, options)
  @config = config
  @logger = logger
  @metrics = metrics
  @provider_name = name

  # Ensure that there is not a nil provider configuration
  @config[:providers] = {} if @config[:providers].nil?
  @config[:providers][@provider_name] = {} if provider_config.nil?

  # Ensure that there is not a nil pool configuration
  @config[:pools] = {} if @config[:pools].nil?

  @provider_options = options
  logger.log('s', "[!] Creating provider '#{name}'")
end

Instance Attribute Details

#loggerObject (readonly)

Helper Methods Global Logger object



11
12
13
# File 'lib/vmpooler/providers/base.rb', line 11

def logger
  @logger
end

#metricsObject (readonly)

Global Metrics object



13
14
15
# File 'lib/vmpooler/providers/base.rb', line 13

def metrics
  @metrics
end

#provider_optionsObject (readonly)

Provider options passed in during initialization



15
16
17
# File 'lib/vmpooler/providers/base.rb', line 15

def provider_options
  @provider_options
end

Instance Method Details

#create_disk(_pool_name, _vm_name, _disk_size) ⇒ Object

inputs

[String]  pool_name  : Name of the pool
[String]  vm_name    : Name of the VM to create the disk on
[Integer] disk_size  : Size of the disk to create in Gigabytes (GB)

returns

[Boolean] : true if success, false if disk could not be created
Raises RuntimeError if the Pool does not exist
Raises RuntimeError if the VM does not exist


167
168
169
# File 'lib/vmpooler/providers/base.rb', line 167

def create_disk(_pool_name, _vm_name, _disk_size)
  raise("#{self.class.name} does not implement create_disk")
end

#create_snapshot(_pool_name, _vm_name, _new_snapshot_name) ⇒ Object

inputs

[String] pool_name         : Name of the pool
[String] new_vmname        : Name of the VM to create the snapshot on
[String] new_snapshot_name : Name of the new snapshot to create

returns

[Boolean] : true if success, false if snapshot could not be created
Raises RuntimeError if the Pool does not exist
Raises RuntimeError if the VM does not exist


179
180
181
# File 'lib/vmpooler/providers/base.rb', line 179

def create_snapshot(_pool_name, _vm_name, _new_snapshot_name)
  raise("#{self.class.name} does not implement create_snapshot")
end

#create_template_delta_disks(_pool) ⇒ Object

inputs

[Hash] pool : Configuration for the pool

returns

nil when successful. Raises error when encountered


227
228
229
# File 'lib/vmpooler/providers/base.rb', line 227

def create_template_delta_disks(_pool)
  raise("#{self.class.name} does not implement create_template_delta_disks")
end

#create_vm(_pool_name, _new_vmname) ⇒ Object

inputs

[String] pool       : Name of the pool
[String] new_vmname : Name to give the new VM

returns

[Hashtable] of the VM as per get_vm
Raises RuntimeError if the pool_name is not supported by the Provider


155
156
157
# File 'lib/vmpooler/providers/base.rb', line 155

def create_vm(_pool_name, _new_vmname)
  raise("#{self.class.name} does not implement create_vm")
end

#destroy_vm(_pool_name, _vm_name) ⇒ Object

inputs

[String] pool_name : Name of the pool
[String] vm_name   : Name of the VM to destroy

returns

[Boolean] : true if success, false on error. Should returns true if the VM is missing


201
202
203
# File 'lib/vmpooler/providers/base.rb', line 201

def destroy_vm(_pool_name, _vm_name)
  raise("#{self.class.name} does not implement destroy_vm")
end

#find_least_used_compatible_host(_pool_name, _vm_name) ⇒ Object

inputs

[String] pool_name : Name of the pool
[String] vm_name   : Name of the VM

returns

[String] : Name of the most appropriate host computer to run this VM.  Useful for load balancing VMs in a cluster
             If this is not a Virtual Machine, it returns the vm_name


110
111
112
# File 'lib/vmpooler/providers/base.rb', line 110

def find_least_used_compatible_host(_pool_name, _vm_name)
  raise("#{self.class.name} does not implement find_least_used_compatible_host")
end

#get_target_datacenter_from_config(_provider_name) ⇒ Object

inputs

[String] provider_name : Name of the provider

returns

Hash of folders


235
236
237
# File 'lib/vmpooler/providers/base.rb', line 235

def get_target_datacenter_from_config(_provider_name)
  raise("#{self.class.name} does not implement get_target_datacenter_from_config")
end

#get_vm(_pool_name, _vm_name) ⇒ Object

inputs

[String] pool_name : Name of the pool
[String] vm_name   : Name of the VM to find

returns

nil if VM doesn't exist
[Hastable] of the VM
 [String] name       : Name of the VM
 [String] hostname   : Name reported by Vmware tools (host.summary.guest.hostName)
 [String] template   : This is the name of template exposed by the API.  It must _match_ the poolname
 [String] poolname   : Name of the pool the VM is located
 [Time]   boottime   : Time when the VM was created/booted
 [String] powerstate : Current power state of a VM.  Valid values (as per vCenter API)
                         - 'PoweredOn','PoweredOff'


145
146
147
# File 'lib/vmpooler/providers/base.rb', line 145

def get_vm(_pool_name, _vm_name)
  raise("#{self.class.name} does not implement get_vm")
end

#get_vm_host(_pool_name, _vm_name) ⇒ Object

inputs

[String]pool_name : Name of the pool
[String] vm_name  : Name of the VM

returns

[String] : Name of the host computer running the vm.  If this is not a Virtual Machine, it returns the vm_name


100
101
102
# File 'lib/vmpooler/providers/base.rb', line 100

def get_vm_host(_pool_name, _vm_name)
  raise("#{self.class.name} does not implement get_vm_host")
end

#global_configObject

returns

[Hashtable] : The entire VMPooler configuration


62
63
64
65
# File 'lib/vmpooler/providers/base.rb', line 62

def global_config
  # This entire VM Pooler config
  @config
end

#migrate_vm(_pool_name, _vm_name, _redis) ⇒ Object

inputs

[String] pool_name      : Name of the pool
[String] vm_name        : Name of the VM to migrate
[Class] redis           : Redis object


128
129
130
# File 'lib/vmpooler/providers/base.rb', line 128

def migrate_vm(_pool_name, _vm_name, _redis)
  raise("#{self.class.name} does not implement migrate_vm")
end

#migrate_vm_to_host(_pool_name, _vm_name, _dest_host_name) ⇒ Object

inputs

[String] pool_name      : Name of the pool
[String] vm_name        : Name of the VM to migrate
[String] dest_host_name : Name of the host to migrate `vm_name` to

returns

[Boolean] : true on success or false on failure


120
121
122
# File 'lib/vmpooler/providers/base.rb', line 120

def migrate_vm_to_host(_pool_name, _vm_name, _dest_host_name)
  raise("#{self.class.name} does not implement migrate_vm_to_host")
end

#nameObject

returns

[String] : Name of the provider service


69
70
71
# File 'lib/vmpooler/providers/base.rb', line 69

def name
  @provider_name
end

#pool_config(pool_name) ⇒ Object

inputs

[String] pool_name : Name of the pool to get the configuration

returns

[Hashtable] : The pools configuration from the config file.  Returns nil if the pool does not exist


40
41
42
43
44
45
46
47
# File 'lib/vmpooler/providers/base.rb', line 40

def pool_config(pool_name)
  # Get the configuration of a specific pool
  @config[:pools].each do |pool|
    return pool if pool['name'] == pool_name
  end

  nil
end

#provided_poolsObject

returns

Array[String] : Array of pool names this provider services


75
76
77
78
79
80
81
# File 'lib/vmpooler/providers/base.rb', line 75

def provided_pools
  list = []
  @config[:pools].each do |pool|
    list << pool['name'] if pool['provider'] == name
  end
  list
end

#provider_configObject

returns

[Hashtable] : This provider's configuration from the config file.  Returns nil if the provider does not exist


51
52
53
54
55
56
57
58
# File 'lib/vmpooler/providers/base.rb', line 51

def provider_config
  @config[:providers].each do |provider|
    # Convert the symbol from the config into a string for comparison
    return (provider[1].nil? ? {} : provider[1]) if provider[0].to_s == @provider_name
  end

  nil
end

#purge_unconfigured_folders(_base_folders, _configured_folders, _whitelist) ⇒ Object



239
240
241
# File 'lib/vmpooler/providers/base.rb', line 239

def purge_unconfigured_folders(_base_folders, _configured_folders, _whitelist)
  raise("#{self.class.name} does not implement purge_unconfigured_folders")
end

#revert_snapshot(_pool_name, _vm_name, _snapshot_name) ⇒ Object

inputs

[String] pool_name     : Name of the pool
[String] new_vmname    : Name of the VM to restore
[String] snapshot_name : Name of the snapshot to restore to

returns

[Boolean] : true if success, false if snapshot could not be revertted
Raises RuntimeError if the Pool does not exist
Raises RuntimeError if the VM does not exist
Raises RuntimeError if the snapshot does not exist


192
193
194
# File 'lib/vmpooler/providers/base.rb', line 192

def revert_snapshot(_pool_name, _vm_name, _snapshot_name)
  raise("#{self.class.name} does not implement revert_snapshot")
end

#vm_exists?(pool_name, vm_name) ⇒ Boolean

inputs

[String] pool_name : Name of the pool
[String] vm_name   : Name of the VM to check if it exists

returns

[Boolean] : true if it exists, false if not

Returns:

  • (Boolean)


219
220
221
# File 'lib/vmpooler/providers/base.rb', line 219

def vm_exists?(pool_name, vm_name)
  !get_vm(pool_name, vm_name).nil?
end

#vm_ready?(_pool_name, _vm_name) ⇒ Boolean

inputs

[String] pool_name : Name of the pool
[String] vm_name   : Name of the VM to check if ready

returns

[Boolean] : true if ready, false if not

Returns:

  • (Boolean)


210
211
212
# File 'lib/vmpooler/providers/base.rb', line 210

def vm_ready?(_pool_name, _vm_name)
  raise("#{self.class.name} does not implement vm_ready?")
end

#vms_in_pool(_pool_name) ⇒ Object

inputs

[String] pool_name : Name of the pool

returns

Array[Hashtable]
  Hash contains:
    'name' => [String] Name of VM


91
92
93
# File 'lib/vmpooler/providers/base.rb', line 91

def vms_in_pool(_pool_name)
  raise("#{self.class.name} does not implement vms_in_pool")
end